/*
 * Centova Cast - http://www.centova.com
 * Copyright 2006-2007, Centova Technologies Inc.
 * 
 *****************************************************************************
 *
 * This file is part of Centova Cast.
 * 
 * Use of this source file is subject to the terms of the Centova Cast License
 * Agreement, a copy of which should have been included in the Centova Cast
 * distribution archive in the file named "LICENSE.txt".  If you did not
 * receive a copy of the Centova Cast License Agreement, please visit the
 * Centova Cast web site at http://www.centova.com and contact a support
 * representative for a copy immediately.
 *
 * Please note that Centova Cast is not an Open Source product; distribution
 * of this file is prohibited under the terms of the Centova Cast License
 * Agreement.
 *
 *
 * DESCRIPTION
 *
 * runascc.c
 *
 * Simple utility to run commands under the CentovaCast userid.  This is 
 * necessary given that you probably don't want to run your streaming servers
 * under the webserver's userid.
 *
 * To install, simply compile this source file (gcc -o runascc runascc.c),
 * chown it to the appropriate UID, and then set the setuid bit.  A Makefile
 * is also provided; to use it, simply run "make all".
 *
 */

// ============================================================================
// CONFIGURATION

// By default, this script will check /usr/bin/php, /usr/local/bin/php, and
// /bin/php (in that order) for the PHP CLI binary.  If yours is located
// elsewhere, you will need to specify the full path and filename to your PHP
// CLI binary below.
#define PHP_PATH ""

//
// END OF CONFIGURATION
// ============================================================================


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>

#define PHP_PATH_COUNT 3
#define PHP_PATHS { "/usr/local/bin/php", "/usr/bin/php", "/bin/php" }

// locates the php cli binary and returns its path in buf
int findphp(char *buf, int bufsize) {
	int i;
	struct stat *filestat;
	char *paths[PHP_PATH_COUNT] = PHP_PATHS;
	
	if (strlen(PHP_PATH)>0) {
		snprintf(buf,bufsize,"%s",PHP_PATH);
		return 0;
	}
	
	for (i=0; i<PHP_PATH_COUNT; i++) {
		filestat = malloc(sizeof (struct stat));
		if (stat(paths[i], filestat)==0) {
			
			snprintf(buf,bufsize,"%s",paths[i]);
			free(filestat);
				
			return 0;
		}
		free(filestat);
	}
	
	return -1;
	
}

int main(int argc, char **argv) {

	char php[1024];
	char *pphp = php;

	char cwd[1024];
	char *pcwd = cwd;
	char ccmanage[1536];
	char *pccmanage = ccmanage;
	char *p = NULL;

	if (argc<1+1) {
		printf("usage: %s exec ccmanage <args...>\n\nExecutes ccmanage.php with the specified arguments.\n",argv[0]);
		exit(EXIT_FAILURE);
	}

	// try to set real UID to EUID
	if (setreuid(geteuid(),-1) == -1) {
		printf("Cannot set real UID to %d\n",(int) geteuid());
		exit(EXIT_FAILURE);
	}
	
	// find ccmanage.php
	if (getcwd(pcwd,sizeof(cwd))==NULL) {
		printf("Path too long\n");
		exit(EXIT_FAILURE);
	}
	if (argv[0][0]=='/') {
		snprintf(pccmanage,sizeof(ccmanage)-32,"%s",argv[0]);
	} else {
		snprintf(pccmanage,sizeof(ccmanage)-32,"%s/%s",pcwd,argv[0]);
	}
	p = strrchr(pccmanage,'/');
	if (p != 0) p[0] = '\0';
	pccmanage = strcat(pccmanage,"/../ccmanage.php");

	// find php
	if (findphp(pphp,sizeof(php)) == -1) {
		printf("Could not locate the PHP CLI binary in any standard locations.\nYou may need to explicitly set the PHP_PATH constant and recompile this program.\n");
		exit(EXIT_FAILURE);
	}

	// call php passing ccmanage.php as the script to execute
	argv[0] = pphp;
	argv[1] = "-q";
	argv[2] = pccmanage;
	
	execv(pphp,argv);

	printf("Error executing %s\n",pphp);
	exit(EXIT_FAILURE);
}
