/*
 * Centova Cast - http://www.centova.com
 * Copyright 2006-2007, Centova Technologies Inc.
 * 
 *****************************************************************************
 *
 * This file is part of Centova Cast.
 * 
 * Use of this source file is subject to the terms of the Centova Cast License
 * Agreement, a copy of which should have been included in the Centova Cast
 * distribution archive in the file named "LICENSE.txt".  If you did not
 * receive a copy of the Centova Cast License Agreement, please visit the
 * Centova Cast web site at http://www.centova.com and contact a support
 * representative for a copy immediately.
 *
 * Please note that Centova Cast is not an Open Source product; distribution
 * of this file is prohibited under the terms of the Centova Cast License
 * Agreement.
 *
 *
 * DESCRIPTION
 *
 * spawn.c
 *
 * This helper binary serves two purposes:
 *
 * 1. Centova Cast needs forked processes to create a new session so that our
 *    servers (and sources) won't be killed when Apache is stopped.
 *    Unfortunately the Process Control functions aren't built into PHP by
 *    default, so we can't rely on those, and we call setsid() here instead.
 * 2. Under Apache 2.x, PHP bug #38915 causes major problems when spawning
 *    server/source processes.  This binary closes all open file handles
 *    after forking to eliminate those problems.
 *
 * Unlike runascc.c, this is not installed as a setuid binary.  Its sole
 * purpose is to fork and execute the specified command under the current
 * uid/gid.  So it should present little if any potential for security issues.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

int main(int argc, char **argv) {
	int pid;
	int i;

	if (argc<2) {
		printf("usage: %s <program_to_fork> [args...]\n",argv[0]);
		exit(EXIT_FAILURE);
	}		
	
	// fork
	pid = fork();
	if (pid<0) {
		// fork failed
		printf("Cannot fork\n");
		exit(EXIT_FAILURE);
	} else if (pid) {
		// we're the parent; output the child's PID so that Centova Cast can
		// monitor it, and exit
		printf("%d\n",pid);
		exit(0);
	}
	// if we reach this point, we're the child
	
	// create a new session
	pid = setsid();	

	// shuffle our arguments back one element
	for (i=0; i<argc-1; i++) {
		argv[i] = argv[i+1];
	}
	argv[argc-1] = NULL;

	// close our file descriptors so that PHP doesn't sit there indefinitely
	// waiting for output, and as a workaround for PHP bug #38915
	for (i=0; i<63; i++) close(i);
	
	// exec
	execv(argv[0],argv);

	// if we reach this point, the exec() failed
	printf("Error executing %s\n",argv[0]);
	exit(EXIT_FAILURE);
}
